<?php
namespace App\Controller;

use App\Templates;
use App\Exception;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class WebService {

    const FAILOVER_EMAIL = "";

    private $util;
    private $integrationService;
    
    private $basePath;
    private $ordersPath;        
    private $processedPath;     
    private $errPath;
    
    private $mailList;
    private $serverName;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util,
                                 \App\Service\IntegrationService $integrationService) {

        $this->connection         = $connection;
        $this->util               = $util;
        $this->integrationService = $integrationService;
    }
    
    public function welcome(Request $request, Response $response) {
        global $CLIENT_DATA;
        
        $config      = $CLIENT_DATA;
        $sql         = "SELECT CDFILIAL FROM FILIAL";
        $result      = $this->connection->fetchAll($sql);
        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'client'       => $config['client'],
            'version'      => $versionInfo['version'],
            'revision'     => $versionInfo['revision'],
            'date'         => $versionInfo['date'],
            'filialCount'  => count($result)
        ]);
    }
    
    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }
    
    public function getProductsAndPrices(Request $request, Response $response) {
        ini_set('zlib.output_compression_level', 1);
        ob_start('ob_gzhandler');

        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Iniciando serviço de exportacao de produtos e precos. \n");
            
            //buscar codigo do subprojeto wedigitek
            $cdSubProj = $this->integrationService->getCdSubProj('WEDI');
            
            //buscar cdfilial e cdcliente para executar a busca dos dados (campo precisa ser criado na ?paravend?)
            $branches = $this->integrationService->getBranches();
            
            $eventCode = $this->integrationService->getLastEventCode();
            $eventCode = $eventCode['CDEVENTO'];
            
            if(empty($eventCode)){
                $eventCode = '0000000000';
            }
            
            $eventDate = $this->integrationService->getLastEventDate();
            $eventDate = $eventDate['DTHREVENTO'];
            
            if(empty($eventDate)){
                $eventDate = '01/01/1900 00:00:00';
            }
            
            foreach($branches as $branch){
                //buscar exportações com erro da filial atual
                $errorProducts = $this->integrationService->getErrorProducts($branch['CDFILIAL']);
                if(empty($errorProducts)){
                    $errorProducts = [];
                }
                
                //sera executado para cada filial retornada, dentro de um foreach. Acrescentar filtro de data pelo campo dtultatu, apenas acima da data da ultima execucao
                //add filtro de cdproduto na lista dos erros no getProductPrices. (buscar produtos que estão apos a data da ultima execução OU que estão dentro dos produtos com erro)
                //na hora de inserir o log na tabela, verificar se o produto é uma retentativa (produto dentro do array de erro). Caso seja atualizar o IDTPEVENTO daquele produto de F para R.
                $productPrices = $this->integrationService->getProductPrices($branch['CDFILIAL'], null, $branch['CDCLIENTE'], $eventDate, $errorProducts);
                
                foreach($productPrices['prices'] as $product){
                    //pegar codigo externo do produto
                    $prodExternalCode = $this->integrationService->getExternalCode($product['CDPRODUTO'], $cdSubProj['CDPROJETO'], $cdSubProj['CDSUBPROJETO'], $branch['CDFILIAL']);
                    
                    //montar o payload de dados e fazer o envio para wedigitek (fazer get dos produtos primeiro para ver se sera novo ou atualização)
                    if(!empty($prodExternalCode)){
                        //rodar o GET
                        $url = 'https://api.wefood.dev/integrations/teknisa/products/' . $prodExternalCode . '?externalUnit=' . $branch['CDFILIALEXT'];
                        
                        $header = array(
                            'Content-Type: application/json',
                            'X-WE-API-KEY: TIv6e3m6F0zfQ2Skk03HQaek2ON3z5cB'
                        );
                        
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Realizando busca de produto {$product['CDARVPROD']}. \n");
                        
                        $curl = curl_init();
                        
                        curl_setopt_array($curl, array(
                                CURLOPT_URL => $url,
                                CURLOPT_RETURNTRANSFER => true,
                                CURLOPT_ENCODING => '',
                                CURLOPT_MAXREDIRS => 10,
                                CURLOPT_TIMEOUT => 15,
                                CURLOPT_FOLLOWLOCATION => true,
                                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                                CURLOPT_CUSTOMREQUEST => 'GET',
                                CURLOPT_HTTPHEADER => $header,
                                CURLOPT_SSL_VERIFYPEER => false
                            )
                        );
                        
                        $response = curl_exec($curl);
                        $response = json_decode($response, true);
                        
                        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                        $curlError = curl_error($curl);
                        
                        curl_close($curl);
                        
                        if($httpCode == 400){
                            //se não, rodar CREATE
                            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Produto {$product['CDARVPROD']} nao encontrado na base Wedigitek. \n");
                            
                            $exportArray = [
                                'name' => [
                                    'pt-BR' => $product['NMPRODUTO'],
                                    'pt-PT' => $product['NMPRODUTO']
                                ],
                                'privacyLevel' => 'restaurantUnit',
                                'restaurantUnit' => [ 
                                    $branch['CDFILIALEXT']
                                ],
                                'reference' => $product['CDARVPROD'],
                                'description' => [
                                    'pt-BR' => $product['NMPRODUTO']
                                ],
                                'ncm' => $product['CDCLASFISC'],
                                'comercialUnit' => $product['SGUNIDADE'],
                                'priceReference' => $product['PRECO']
                            ];

            			    if(!empty($product['CDBARPRODUTO'])){
                				$exportArray['ean'] = $product['CDBARPRODUTO'];
                				$exportArray['code'] = $product['CDBARPRODUTO'];
            			    }
                            
                            if(!empty($product['NMPRODINGLES'])){
                                $exportArray['name']['en'] = $product['NMPRODINGLES'];
                                $exportArray['description']['en'] = $product['NMPRODINGLES'];
                            }
                            
                            if(!empty($product['VRPESOUNID'])){
                                $exportArray['co2Kg'] = $product['VRPESOUNID'];
                            }
                            
                            $url = 'https://api.wefood.dev/integrations/teknisa/products';
                            
                            $header = array(
                                'Content-Type: application/json',
                                'X-WE-API-KEY: TIv6e3m6F0zfQ2Skk03HQaek2ON3z5cB'
                            );
                            
                            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Processando exportacao de produto {$exportArray['reference']}. \n");
                            
                            $curl = curl_init();
                            
                            curl_setopt_array($curl, array(
                                    CURLOPT_URL => $url,
                                    CURLOPT_RETURNTRANSFER => true,
                                    CURLOPT_ENCODING => '',
                                    CURLOPT_MAXREDIRS => 10,
                                    CURLOPT_TIMEOUT => 15,
                                    CURLOPT_FOLLOWLOCATION => true,
                                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                                    CURLOPT_CUSTOMREQUEST => 'POST',
                                    CURLOPT_POSTFIELDS => json_encode($exportArray),
                                    CURLOPT_HTTPHEADER => $header,
                                    CURLOPT_SSL_VERIFYPEER => false
                                )
                            );
                            
                            $response = curl_exec($curl);
                            $response = json_decode($response, true);
                            
                            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                            $curlError = curl_error($curl);
                            
                            curl_close($curl);
                            
                            if (!empty($curlError) || (($httpCode != 200) && ($httpCode != 201)) || (stripos($response, "<html>")) || (stripos($response, "Fatal error"))) {
                                $fileName = "EXP-".date("H-i-s") . ".txt";
                    
                                $url_path = $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                                    'eventTime' => date("d/m/Y H:i:s"),
                                    'header'    => $header,
                                    'url'       => $url,
                                    'method'    => 'POST',
                                    'params'    => json_encode($exportArray),
                                    'httpCode'  => $httpCode,
                                    'error'     => $curlError,
                                    'response'  => json_encode($response)
                                ]));
                                
                                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na exportacao de produto {$exportArray['reference']}. \n");
                                $this->integrationService->saveProductExpLog(str_pad((((int)$eventCode) + 1), 10, '0', STR_PAD_LEFT), $branch['CDFILIAL'], $product['CDPRODUTO'], 'F', "Erro na exportacao de produto {$exportArray['reference']}", $url_path);
                                foreach($errorProducts as $errorProduct){
                                    if($errorProduct === $product['CDPRODUTO']){
                                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Realizando atualização do log de retentativa para o produto {$exportArray['reference']}. \n");
                                        $this->integrationService->updateErrorLog($product['CDPRODUTO'], $branch['CDFILIAL']);
                                    }
                                }
                            } else{
                                //inserir _id no campo CDPRODINTE
                                $this->integrationService->setCdProdInte($response['_id'], $product['CDPRODUTO'], $cdSubProj['CDPROJETO'], $cdSubProj['CDSUBPROJETO'], $branch['CDFILIAL']);
                                
                                //salvar em log na base (tabela precisa ser criada)
                                $fileName = "{$branch['CDFILIAL']}-{$product['CDPRODUTO']}-" . date("H-i-s") . ".json";
                                
                                $url_path = $this->util->writeLog("product_export", $fileName, $response);
                                
                                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Exportacao de produto {$exportArray['reference']} realizada com sucesso. \n");
                                $this->integrationService->saveProductExpLog(str_pad((((int)$eventCode) + 1), 10, '0', STR_PAD_LEFT), $branch['CDFILIAL'], $product['CDPRODUTO'], 'E', "Exportacao de produto {$exportArray['reference']} realizada com sucesso", $url_path);
                                foreach($errorProducts as $errorProduct){
                                    if($errorProduct === $product['CDPRODUTO']){
                                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Realizando atualização do log de retentativa para o produto {$exportArray['reference']}. \n");
                                        $this->integrationService->updateErrorLog($product['CDPRODUTO'], $branch['CDFILIAL']);
                                    }
                                }
                            }
                        } else if (!empty($curlError) || (($httpCode != 200) && ($httpCode != 201)) || (stripos($response, "<html>")) || (stripos($response, "Fatal error"))) {
                            $fileName = "EXP-".date("H-i-s") . ".txt";
                
                            $url_path = $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                                'eventTime' => date("d/m/Y H:i:s"),
                                'header'    => $header,
                                'url'       => $url,
                                'method'    => 'GET',
                                'params'    => [],
                                'httpCode'  => $httpCode,
                                'error'     => $curlError,
                                'response'  => json_encode($response)
                            ]));
                            
                            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na busca de produto {$product['CDARVPROD']}. \n");
                            $this->integrationService->saveProductExpLog(str_pad((((int)$eventCode) + 1), 10, '0', STR_PAD_LEFT), $branch['CDFILIAL'], $product['CDPRODUTO'], 'F', "Erro na busca de produto {$product['CDARVPROD']}", $url_path);
                            foreach($errorProducts as $errorProduct){
                                if($errorProduct === $product['CDPRODUTO']){
                                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Realizando atualização do log de retentativa para o produto {$product['CDARVPROD']}. \n");
                                    $this->integrationService->updateErrorLog($product['CDPRODUTO'], $branch['CDFILIAL']);
                                }
                            }
                        } else{
                            if(!empty($response['_id'])){
                                //se retornar, rodar UPDATE
                                $exportArray = [
                                    'name' => [
                                        'pt-BR' => $product['NMPRODUTO'],
                                        'pt-PT' => $product['NMPRODUTO']
                                    ],
                                    'privacyLevel' => 'restaurantUnit',
                                    'restaurantUnit' => [ 
                                        $branch['CDFILIALEXT']
                                    ],
                                    'reference' => $product['CDARVPROD'],
                                    'description' => [
                                        'pt-BR' => $product['NMPRODUTO']
                                    ],
                                    'ncm' => $product['CDCLASFISC'],
                                    'comercialUnit' => $product['SGUNIDADE'],
                                    'priceReference' => $product['PRECO']
                                ];

				                if(!empty($product['CDBARPRODUTO'])){
                             	    $exportArray['ean'] = $product['CDBARPRODUTO'];
                                    $exportArray['code'] = $product['CDBARPRODUTO'];
                                }
                                
                                if(!empty($product['NMPRODINGLES'])){
                                    $exportArray['name']['en'] = $product['NMPRODINGLES'];
                                    $exportArray['description']['en'] = $product['NMPRODINGLES'];
                                }
                                
                                if(!empty($product['VRPESOUNID'])){
                                    $exportArray['co2Kg'] = $product['VRPESOUNID'];
                                }
                                
                                $url = 'https://api.wefood.dev/integrations/teknisa/products/' . $prodExternalCode . '?externalUnit=' . $branch['CDFILIALEXT'];
                                
                                $header = array(
                                    'Content-Type: application/json',
                                    'X-WE-API-KEY: TIv6e3m6F0zfQ2Skk03HQaek2ON3z5cB'
                                );
                                
                                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Processando atualizacao de produto {$exportArray['reference']}. \n");
                                
                                $curl = curl_init();
                                
                                curl_setopt_array($curl, array(
                                        CURLOPT_URL => $url,
                                        CURLOPT_RETURNTRANSFER => true,
                                        CURLOPT_ENCODING => '',
                                        CURLOPT_MAXREDIRS => 10,
                                        CURLOPT_TIMEOUT => 15,
                                        CURLOPT_FOLLOWLOCATION => true,
                                        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                                        CURLOPT_CUSTOMREQUEST => 'PUT',
                                        CURLOPT_POSTFIELDS => json_encode($exportArray),
                                        CURLOPT_HTTPHEADER => $header,
                                        CURLOPT_SSL_VERIFYPEER => false
                                    )
                                );
                                
                                $response = curl_exec($curl);
                                $response = json_decode($response, true);
                                
                                $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                                $curlError = curl_error($curl);
                                
                                curl_close($curl);
                                
                                if (!empty($curlError) || (($httpCode != 200) && ($httpCode != 201)) || (stripos($response, "<html>")) || (stripos($response, "Fatal error"))) {
                                    $fileName = "EXP-".date("H-i-s") . ".txt";
                        
                                    $url_path = $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                                        'eventTime' => date("d/m/Y H:i:s"),
                                        'header'    => $header,
                                        'url'       => $url,
                                        'method'    => 'PUT',
                                        'params'    => json_encode($exportArray),
                                        'httpCode'  => $httpCode,
                                        'error'     => $curlError,
                                        'response'  => json_encode($response)
                                    ]));
                                    
                                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na atualizacao de produto {$exportArray['reference']}. \n");
                                    $this->integrationService->saveProductExpLog(str_pad((((int)$eventCode) + 1), 10, '0', STR_PAD_LEFT), $branch['CDFILIAL'], $product['CDPRODUTO'], 'F', "Erro na atualizacao de produto {$exportArray['reference']}", $url_path);
                                    foreach($errorProducts as $errorProduct){
                                        if($errorProduct === $product['CDPRODUTO']){
                                            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Realizando atualização do log de retentativa para o produto {$exportArray['reference']}. \n");
                                            $this->integrationService->updateErrorLog($product['CDPRODUTO'], $branch['CDFILIAL']);
                                        }
                                    }
                                } else{
                                    //salvar em log na base (tabela precisa ser criada)
                                    $fileName = "{$branch['CDFILIAL']}-{$product['CDPRODUTO']}-" . date("H-i-s") . ".json";
                                
                                    $url_path = $this->util->writeLog("product_update", $fileName, $response);
                                    
                                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Atualizacao de produto {$exportArray['reference']} realizada com sucesso. \n");
                                    $this->integrationService->saveProductExpLog(str_pad((((int)$eventCode) + 1), 10, '0', STR_PAD_LEFT), $branch['CDFILIAL'], $product['CDPRODUTO'], 'A', "Atualizacao de produto {$exportArray['reference']} realizada com sucesso", $url_path);
                                    foreach($errorProducts as $errorProduct){
                                        if($errorProduct === $product['CDPRODUTO']){
                                            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Realizando atualização do log de retentativa para o produto {$exportArray['reference']}. \n");
                                            $this->integrationService->updateErrorLog($product['CDPRODUTO'], $branch['CDFILIAL']);
                                        }
                                    }
                                }
                            }
                        }
                    } else{
                        //rodar CREATE
                        $exportArray = [
                            'name' => [
                                'pt-BR' => $product['NMPRODUTO'],
                                'pt-PT' => $product['NMPRODUTO']
                            ],
                            'privacyLevel' => 'restaurantUnit',
                            'restaurantUnit' => [ 
                                $branch['CDFILIALEXT']
                            ],
                            'reference' => $product['CDARVPROD'],
                            'description' => [
                                'pt-BR' => $product['NMPRODUTO']
                            ],
                            'ncm' => $product['CDCLASFISC'],
                            'comercialUnit' => $product['SGUNIDADE'],
                            'priceReference' => $product['PRECO']
                        ];

			            if(!empty($product['CDBARPRODUTO'])){
                            $exportArray['ean'] = $product['CDBARPRODUTO'];
                            $exportArray['code'] = $product['CDBARPRODUTO'];
                        }
                        
                        if(!empty($product['NMPRODINGLES'])){
                            $exportArray['name']['en'] = $product['NMPRODINGLES'];
                            $exportArray['description']['en'] = $product['NMPRODINGLES'];
                        }
                        
                        if(!empty($product['VRPESOUNID'])){
                            $exportArray['co2Kg'] = $product['VRPESOUNID'];
                        }
                        
                        $url = 'https://api.wefood.dev/integrations/teknisa/products';
                        
                        $header = array(
                            'Content-Type: application/json',
                            'X-WE-API-KEY: TIv6e3m6F0zfQ2Skk03HQaek2ON3z5cB'
                        );
                        
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Processando exportacao de produto {$exportArray['reference']}. \n");
                        
                        $curl = curl_init();
                        
                        curl_setopt_array($curl, array(
                                CURLOPT_URL => $url,
                                CURLOPT_RETURNTRANSFER => true,
                                CURLOPT_ENCODING => '',
                                CURLOPT_MAXREDIRS => 10,
                                CURLOPT_TIMEOUT => 15,
                                CURLOPT_FOLLOWLOCATION => true,
                                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                                CURLOPT_CUSTOMREQUEST => 'POST',
                                CURLOPT_POSTFIELDS => json_encode($exportArray),
                                CURLOPT_HTTPHEADER => $header,
                                CURLOPT_SSL_VERIFYPEER => false
                            )
                        );
                        
                        $response = curl_exec($curl);
                        $response = json_decode($response, true);
                        
                        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                        $curlError = curl_error($curl);
                        
                        curl_close($curl);
                        
                        if (!empty($curlError) || (($httpCode != 200) && ($httpCode != 201)) || (stripos($response, "<html>")) || (stripos($response, "Fatal error"))) {
                            $fileName = "EXP-".date("H-i-s") . ".txt";
                
                            $url_path = $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                                'eventTime' => date("d/m/Y H:i:s"),
                                'header'    => $header,
                                'url'       => $url,
                                'method'    => 'POST',
                                'params'    => json_encode($exportArray),
                                'httpCode'  => $httpCode,
                                'error'     => $curlError,
                                'response'  => json_encode($response)
                            ]));
                            
                            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na exportacao de produto {$exportArray['reference']}. \n");
                            $this->integrationService->saveProductExpLog(str_pad((((int)$eventCode) + 1), 10, '0', STR_PAD_LEFT), $branch['CDFILIAL'], $product['CDPRODUTO'], 'F', "Erro na exportacao de produto {$exportArray['reference']}", $url_path);
                            foreach($errorProducts as $errorProduct){
                                if($errorProduct === $product['CDPRODUTO']){
                                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Realizando atualização do log de retentativa para o produto {$exportArray['reference']}. \n");
                                    $this->integrationService->updateErrorLog($product['CDPRODUTO'], $branch['CDFILIAL']);
                                }
                            }
                        } else{
                            //inserir _id no campo CDPRODINTE
                            $this->integrationService->setCdProdInte($response['_id'], $product['CDPRODUTO'], $cdSubProj['CDPROJETO'], $cdSubProj['CDSUBPROJETO'], $branch['CDFILIAL']);
                            
                            //salvar em log na base (tabela precisa ser criada)
                            $fileName = "{$branch['CDFILIAL']}-{$product['CDPRODUTO']}-" . date("H-i-s") . ".json";
                                
                            $url_path = $this->util->writeLog("product_export", $fileName, $response);
                            
                            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Exportacao de produto {$exportArray['reference']} realizada com sucesso. \n");
                            $this->integrationService->saveProductExpLog(str_pad((((int)$eventCode) + 1), 10, '0', STR_PAD_LEFT), $branch['CDFILIAL'], $product['CDPRODUTO'], 'E', "Exportacao de produto {$exportArray['reference']} realizada com sucesso", $url_path);
                            foreach($errorProducts as $errorProduct){
                                if($errorProduct === $product['CDPRODUTO']){
                                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Realizando atualização do log de retentativa para o produto {$exportArray['reference']}. \n");
                                    $this->integrationService->updateErrorLog($product['CDPRODUTO'], $branch['CDFILIAL']);
                                }
                            }
                        }
                    }
                    $eventCode = str_pad((((int)$eventCode) + 1), 10, '0', STR_PAD_LEFT);
                    sleep(2);
                }
            }
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Serviço de exportacao de produtos e precos finalizado com sucesso. \n");
            
            return json_encode([
                'error' => false,
                'message' => 'Envio de produtos e precos executado com sucesso'
            ]);
        } catch (\Exception $e) {
            $message = 'Erro ao executar o envio de produtos e precos. '.$e->getMessage();
            return json_encode([
                'error' => true,
                'message' => $message
            ]);
        }
    
    }
    
    public function deleteFolder(Request $request, Response $response){
        $req = $request->getParsedBody();
        
        if ($req["auth"] != "81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E") 
                throw new \Exception("Auth inválido.", 666);
        
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);
        
        $deletePath = $config['saleFiles'].'/'.$req['branch'].'/'.$req['folder'];
        
        if(file_exists($deletePath)){
            exec('rm -rf '.$deletePath);
        }
    }
    
    private function deleteFiles(){
        /*Removendo os arquivos com mais de 3 meses*/
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);
        
        $limitTime = time() - 8035200; // timestamp atual - 3 meses (60 * 60 * 24 * 93)
        
        $deletePathArr = [];
        
        if(file_exists($config['saleFiles'])) {
            if($branchDirHandle = opendir($config['saleFiles'])) { //verifica se foi possivel abrir o diretorio
                while (($branchFolder = readdir($branchDirHandle)) !== false) {
                    if ($branchFolder != "." && $branchFolder != "..") { // desconsiderar . e ..
                        if(file_exists($config['saleFiles'].'/'.$branchFolder)) {
                            if($salesDirHandle = opendir($config['saleFiles'].'/'.$branchFolder)) { //verifica se foi possivel abrir o diretorio
                                while (($salesFolder = readdir($salesDirHandle)) !== false) {
                                    if ($salesFolder != "." && $salesFolder != "..") { // desconsiderar . e ..
                                        $folderTimestamp = strtotime("20".$salesFolder);
                                        if(!$folderTimestamp) continue; // se não for uma data, ele pula a interação
                                        
                                        if($folderTimestamp <= $limitTime) {
                                            $deletePath = $config['saleFiles'].'/'.$branchFolder.'/'.$salesFolder.'/';
                                            if(file_exists($deletePath)){
                                                array_push($deletePathArr, $deletePath);
                                            }
                                        }
                                    }
                                }
                                closedir($salesDirHandle);
                            }
                        }
                    }
                }
                closedir($branchDirHandle);
            }
        }
        
        foreach($deletePathArr as $folderToDelete){
            if(file_exists($deletePath)){
                exec('rm -rf '.$folderToDelete);   
            }
        }
        //fim da rotina de limpeza
    }

    private function writeEventsLog($events) {
        foreach($events as $event) {

            $reference = $event['correlationId'];
            $eventCode = $event['code'];

            $fileName = round(microtime(true)*1000) . "-$eventCode.json";
            $this->util->writeLog("events", $fileName, $event);
        }
    }

    private function getClientName() {
        global $CLIENT_NAME;
        return $CLIENT_NAME;
    }
}
