<?php
namespace App\Factory;

use App\Exception;

class IntegrationFactory {
    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function getBranches() {
        $sql = "
            SELECT P.CDFILIAL, P.CDCLIENTE, F.CDFILIALEXT
              FROM PARAVEND P
              JOIN FILIAL F
                ON F.CDFILIAL = P.CDFILIAL
             WHERE IDUTWEDIPRICES = 'S'
        ";

        return $this->connection->fetchAll($sql);
    }
    
    public function getLastEventCode() {
        $sql = "
            SELECT MAX(CDEVENTO) AS CDEVENTO
              FROM LOGEXPPROD
        ";

        return $this->connection->fetchAssoc($sql);
    }
    
    public function getLastEventDate() {
        $sql = "
            SELECT TO_CHAR(MAX(DTHREVENTO), 'dd/mm/yyyy hh24:mi:ss') AS DTHREVENTO
              FROM LOGEXPPROD
             WHERE IDTPEVENTO <> 'F'
        ";

        return $this->connection->fetchAssoc($sql);
    }
    
    public function saveProductExpLog($CDEVENTO, $CDFILIAL, $CDPRODUTO, $IDTPEVENTO, $DSEVENTO, $URLEVENTO) {
        $sql = "
            INSERT INTO LOGEXPPROD
            (CDEVENTO, CDFILIAL, DTHREVENTO, CDPRODUTO, IDTPEVENTO, DSEVENTO, URLEVENTO)
            VALUES
            (:CDEVENTO, :CDFILIAL, SYSDATE, :CDPRODUTO, :IDTPEVENTO, :DSEVENTO, :URLEVENTO)
        ";
        
        $params = array(
            ":CDEVENTO" => $CDEVENTO,
            ":CDFILIAL" => $CDFILIAL,
            ":CDPRODUTO" => $CDPRODUTO,
            ":IDTPEVENTO" => $IDTPEVENTO,
            ":DSEVENTO" => $DSEVENTO,
            ":URLEVENTO" => $URLEVENTO
        );

        return $this->connection->executeQuery($sql, $params);
    }
	
	public function getProductPrices($CDFILIAL, $CDLOJA, $CDCLIENTE, $eventDate, $errorProducts) {
        try {

            $date = date('Y-m-d H:i:s');

            if(!empty($CDCLIENTE)) {
                // TABELA CLIENTE POR FILIAL
                $sql = "
                    SELECT CDTABEPREC, CDCFILTABPRE
                      FROM CLIENFILIAL
                     WHERE CDCLIENTE = :CDCLIENTE
                       AND CDFILIAL = :CDFILIAL
                ";
                
                $params = array(
                    ":CDFILIAL" => $CDFILIAL,
                    ":CDCLIENTE" => $CDCLIENTE
                );

                $cliBranchTabePrec = $this->connection->fetchAssoc($sql, $params);
                
                if(!empty($cliBranchTabePrec['CDTABEPREC'])){
                    $sql = "
                        SELECT DTINIVGPREC
                          FROM TABEVEND
                         WHERE (CDFILIAL = :CDFILIAL)
                           AND (CDTABEPREC = :CDTABEPREC)
                           AND (TO_DATE(:DATA, 'yyyy-mm-dd hh24:mi:ss') BETWEEN DTINIVGPREC AND DTFINVGPREC)
                    ";
                    
                    $params = array(
                        ":CDFILIAL" => $CDFILIAL,
                        ":CDTABEPREC" => $cliBranchTabePrec['CDTABEPREC'],
                        ":DATA" => $date
                    );

                    $cliBranchTabePrecVig = $this->connection->fetchAssoc($sql, $params);

                    if(!empty($cliBranchTabePrecVig['DTINIVGPREC'])){
                        $sql = "
                            SELECT I.CDPRODUTO,
                                   P.NMPRODUTO,
                                   P.NMPRODINGLES,
                                   P.CDARVPROD,
                                   P.CDPRODINTE,
                                   P.VRPESOUNID,
                                   P.CDBARPRODUTO,
                                   P.CDCLASFISC,
                                   P.SGUNIDADE,
                                   I.VRPRECITEM AS PRECO
                              FROM ITEMPRECO I
                              JOIN PRODUTO P
                                ON I.CDPRODUTO = P.CDPRODUTO
                             WHERE (I.CDFILIAL = :CDFILIAL)
                               AND (I.CDTABEPREC = :CDTABEPREC)
                               AND (((I.DTULTATUITPR > TO_DATE(:DTHREVENTO, 'dd/mm/yyyy hh24:mi:ss')) OR I.DTULTATUITPR IS NULL) OR
                                    ((P.DTULTATUPROD > TO_DATE(:DTHREVENTO, 'dd/mm/yyyy hh24:mi:ss')) OR P.DTULTATUPROD IS NULL) OR
                                    (I.CDPRODUTO IN :CDPRODUTO))
			                   AND I.DTINIVGPREC = :DTINIVGPREC
                               AND I.VRPRECITEM > 0
                               AND I.VRPRECITEM IS NOT NULL
                        ";
                        
                        $paramsType = array(':CDPRODUTO' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY);
                        
                        $params = array(
                            ":CDFILIAL" => $CDFILIAL,
                            ":CDTABEPREC" => $cliBranchTabePrec['CDTABEPREC'],
                            ":DTHREVENTO" => $eventDate,
                            ":CDPRODUTO" => $errorProducts,
			                ":DTINIVGPREC" => $cliBranchTabePrecVig['DTINIVGPREC']
                        );
                        $cliBranchTablePrices = $this->connection->fetchAll($sql, $params, $paramsType);
                    } else{
                        $cliBranchTablePrices = [];
                    }
                }  else{
                    $cliBranchTablePrices = [];
                }


                // TABELA CLIENTE
                $sql = "
                    SELECT CDTABEPREC, CDFILTABPREC
                      FROM CLIENTE
                     WHERE (CDCLIENTE = :CDCLIENTE)
                ";
                
                $params = array(
                    ":CDCLIENTE" => $CDCLIENTE
                );

                $clientTabePrec = $this->connection->fetchAssoc($sql, $params);

                if(!empty($clientTabePrec['CDTABEPREC'])){
                    $sql = "
                        SELECT DTINIVGPREC
                          FROM TABEVEND
                         WHERE (CDFILIAL = :CDFILIAL)
                           AND (CDTABEPREC = :CDTABEPREC)
                           AND (TO_DATE(:DATA, 'yyyy-mm-dd hh24:mi:ss') BETWEEN DTINIVGPREC AND DTFINVGPREC)
                    ";
                    
                    $params = array(
                        ":CDFILIAL" => $CDFILIAL,
                        ":CDTABEPREC" => $clientTabePrec['CDTABEPREC'],
                        ":DATA" => $date
                    );

                    $clientTabePrecVig = $this->connection->fetchAssoc($sql, $params);

                    if(!empty($clientTabePrecVig['DTINIVGPREC'])){
                        $sql = "
                            SELECT I.CDPRODUTO,
                                   P.NMPRODUTO,
                                   P.NMPRODINGLES,
                                   P.CDARVPROD,
                                   P.CDPRODINTE,
                                   P.VRPESOUNID,
                                   P.CDBARPRODUTO,
                                   P.CDCLASFISC,
                                   P.SGUNIDADE,
                                   I.VRPRECITEM AS PRECO
                              FROM ITEMPRECO I
                              JOIN PRODUTO P
                                ON I.CDPRODUTO = P.CDPRODUTO
                             WHERE (I.CDFILIAL = :CDFILIAL)
                               AND (I.CDTABEPREC = :CDTABEPREC)
                               AND (((I.DTULTATUITPR > TO_DATE(:DTHREVENTO, 'dd/mm/yyyy hh24:mi:ss')) OR I.DTULTATUITPR IS NULL) OR
                                    ((P.DTULTATUPROD > TO_DATE(:DTHREVENTO, 'dd/mm/yyyy hh24:mi:ss')) OR P.DTULTATUPROD IS NULL) OR
                                    (I.CDPRODUTO IN :CDPRODUTO))
			                   AND I.DTINIVGPREC = :DTINIVGPREC
                               AND I.VRPRECITEM > 0
                               AND I.VRPRECITEM IS NOT NULL
                        ";
                        
                        $paramsType = array(':CDPRODUTO' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY);
                        
                        $params = array(
                            ":CDFILIAL" => $CDFILIAL,
                            ":CDTABEPREC" => $clientTabePrec['CDTABEPREC'],
                            ":DTHREVENTO" => $eventDate,
                            ":CDPRODUTO" => $errorProducts,
			                ":DTINIVGPREC" => $clientTabePrecVig['DTINIVGPREC']
                        );
                        $clientTablePrices = $this->connection->fetchAll($sql, $params, $paramsType);
                    } else{
                        $clientTablePrices = [];
                    }
                }  else{
                    $clientTablePrices = [];
                }
            } else{
                $cliBranchTablePrices = [];
                $clientTablePrices = [];
            }


            // TABELA POR LOJA
            $sql = "
                SELECT CDTABEPREC
                  FROM LOJA
                 WHERE (CDFILIAL = :CDFILIAL)
                   AND (CDLOJA = :CDLOJA)
            ";
            
            $params = array(
                ":CDFILIAL" => $CDFILIAL,
                ":CDLOJA" => $CDLOJA
            );

            $storeTabePrec = $this->connection->fetchAssoc($sql, $params);

            if(!empty($storeTabePrec['CDTABEPREC'])){
                $sql = "
                    SELECT DTINIVGPREC
                      FROM TABEVEND
                     WHERE (CDFILIAL = :CDFILIAL)
                       AND (CDTABEPREC = :CDTABEPREC)
                       AND (TO_DATE(:DATA, 'yyyy-mm-dd hh24:mi:ss') BETWEEN DTINIVGPREC AND DTFINVGPREC)
                ";
                
                $params = array(
                    ":CDFILIAL" => $CDFILIAL,
                    ":CDTABEPREC" => $storeTabePrec['CDTABEPREC'],
                    ":DATA" => $date
                );

                $storeTabePrecVig = $this->connection->fetchAssoc($sql, $params);

                if(!empty($storeTabePrecVig['DTINIVGPREC'])){
                    $sql = "
                        SELECT I.CDPRODUTO,
                               P.NMPRODUTO,
                               P.NMPRODINGLES,
                               P.CDARVPROD,
                               P.CDPRODINTE,
                               P.VRPESOUNID,
                               P.CDBARPRODUTO,
                               P.CDCLASFISC,
                               P.SGUNIDADE,
                               I.VRPRECITEM AS PRECO
                          FROM ITEMPRECO I
                          JOIN PRODUTO P
                            ON I.CDPRODUTO = P.CDPRODUTO
                         WHERE (I.CDFILIAL = :CDFILIAL)
                           AND (I.CDTABEPREC = :CDTABEPREC)
                           AND (((I.DTULTATUITPR > TO_DATE(:DTHREVENTO, 'dd/mm/yyyy hh24:mi:ss')) OR I.DTULTATUITPR IS NULL) OR
                                ((P.DTULTATUPROD > TO_DATE(:DTHREVENTO, 'dd/mm/yyyy hh24:mi:ss')) OR P.DTULTATUPROD IS NULL) OR
                                (I.CDPRODUTO IN :CDPRODUTO))
			               AND I.DTINIVGPREC = :DTINIVGPREC
                           AND I.VRPRECITEM > 0
                           AND I.VRPRECITEM IS NOT NULL
                    ";
                    
                    $paramsType = array(':CDPRODUTO' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY);
                    
                    $params = array(
                        ":CDFILIAL" => $CDFILIAL,
                        ":CDTABEPREC" => $storeTabePrec['CDTABEPREC'],
                        ":DTHREVENTO" => $eventDate,
                        ":CDPRODUTO" => $errorProducts,
			            ":DTINIVGPREC" => $storeTabePrecVig['DTINIVGPREC']
                    );
                    $storeTablePrices = $this->connection->fetchAll($sql, $params, $paramsType);
                } else{
                    $storeTablePrices = [];
                }
            }  else{
                $storeTablePrices = [];
            }


            // TABELA POR FILIAL
            $sql = "
                SELECT CDTABEPREC, IDPRECDIFCOM
                  FROM PARAVEND
                 WHERE (CDFILIAL = :CDFILIAL)
            ";
            
            $params = array(
                ":CDFILIAL" => $CDFILIAL
            );

            $branchTabePrec = $this->connection->fetchAssoc($sql, $params);

            if(!empty($branchTabePrec['CDTABEPREC'])){
                $sql = "
                    SELECT DTINIVGPREC
                      FROM TABEVEND
                     WHERE (CDFILIAL = :CDFILIAL)
                       AND (CDTABEPREC = :CDTABEPREC)
                       AND (TO_DATE(:DATA, 'yyyy-mm-dd hh24:mi:ss') BETWEEN DTINIVGPREC AND DTFINVGPREC)
                ";
                
                $params = array(
                    ":CDFILIAL" => $CDFILIAL,
                    ":CDTABEPREC" => $branchTabePrec['CDTABEPREC'],
                    ":DATA" => $date
                );

                $branchTabePrecVig = $this->connection->fetchAssoc($sql, $params);

                if(!empty($branchTabePrecVig['DTINIVGPREC'])){
                    $sql = "
                        SELECT I.CDPRODUTO,
                               P.NMPRODUTO,
                               P.NMPRODINGLES,
                               P.CDARVPROD,
                               P.CDPRODINTE,
                               P.VRPESOUNID,
                               P.CDBARPRODUTO,
                               P.CDCLASFISC,
                               P.SGUNIDADE,
                               I.VRPRECITEM AS PRECO
                          FROM ITEMPRECO I
                          JOIN PRODUTO P
                            ON I.CDPRODUTO = P.CDPRODUTO
                         WHERE (I.CDFILIAL = :CDFILIAL)
                           AND (I.CDTABEPREC = :CDTABEPREC)
                           AND (((I.DTULTATUITPR > TO_DATE(:DTHREVENTO, 'dd/mm/yyyy hh24:mi:ss')) OR I.DTULTATUITPR IS NULL) OR
                                ((P.DTULTATUPROD > TO_DATE(:DTHREVENTO, 'dd/mm/yyyy hh24:mi:ss')) OR P.DTULTATUPROD IS NULL) OR
                                (I.CDPRODUTO IN :CDPRODUTO))
			               AND I.DTINIVGPREC = :DTINIVGPREC
                           AND I.VRPRECITEM > 0
                           AND I.VRPRECITEM IS NOT NULL
                    ";
                    
                    $paramsType = array(':CDPRODUTO' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY);
                    
                    $params = array(
                        ":CDFILIAL" => $CDFILIAL,
                        ":CDTABEPREC" => $branchTabePrec['CDTABEPREC'],
                        ":DTHREVENTO" => $eventDate,
                        ":CDPRODUTO" => $errorProducts,
			            ":DTINIVGPREC" => $branchTabePrecVig['DTINIVGPREC']
                    );
                    $branchTablePrices = $this->connection->fetchAll($sql, $params, $paramsType);
                } else{
                    $branchTablePrices = [];
                }
            }  else{
                $branchTablePrices = [];
            }


            // TABELA POR GERAL
            $sql = "
                SELECT T.CDTABEPREC, T.DTINIVGPREC
                  FROM TABEVENDGER T, FILITABGER FT
                 WHERE (FT.CDFILIAL = :CDFILIAL)
                   AND (T.CDTABEPREC = FT.CDTABEPREC)
                   AND (TO_DATE(:DATA, 'yyyy-mm-dd hh24:mi:ss') BETWEEN DTINIVGPREC AND DTFINVGPREC)
            ";
            
            $params = array(
                ":CDFILIAL" => $CDFILIAL,
                ":DATA" => $date
            );

            $generalTabePrec = $this->connection->fetchAssoc($sql, $params);

            if(!empty($generalTabePrec['CDTABEPREC'])){
                $sql = "
                    SELECT I.CDPRODUTO,
                           P.NMPRODUTO,
                           P.NMPRODINGLES,
                           P.CDARVPROD,
                           P.CDPRODINTE,
                           P.VRPESOUNID,
                           P.CDBARPRODUTO,
                           P.CDCLASFISC,
                           P.SGUNIDADE,
                           I.VRPRECITEM AS PRECO
                      FROM ITEMPRECOGER I
                      JOIN PRODUTO P
                        ON I.CDPRODUTO = P.CDPRODUTO
                     WHERE (I.CDTABEPREC = :CDTABEPREC)
                       AND (((I.DTULTATU > TO_DATE(:DTHREVENTO, 'dd/mm/yyyy hh24:mi:ss')) OR I.DTULTATU IS NULL) OR
                            ((P.DTULTATUPROD > TO_DATE(:DTHREVENTO, 'dd/mm/yyyy hh24:mi:ss')) OR P.DTULTATUPROD IS NULL) OR
                            (I.CDPRODUTO IN :CDPRODUTO))
                       AND I.VRPRECITEM > 0
                       AND I.VRPRECITEM IS NOT NULL
                ";
                
                $paramsType = array(':CDPRODUTO' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY);
                
                $params = array(
                    ":CDTABEPREC" => $generalTabePrec['CDTABEPREC'],
                    ":DTHREVENTO" => $eventDate,
                    ":CDPRODUTO" => $errorProducts
                );
                $generalTablePrices = $this->connection->fetchAll($sql, $params, $paramsType);
            }  else{
                $generalTablePrices = [];
            }
            
            // CALCULO PRECO BASE FINAL
            $finalProductPrices = [];

            foreach($cliBranchTablePrices as $cliBranchProduct){
                $finalProductPrices[$cliBranchProduct['CDPRODUTO']] = $cliBranchProduct;
                $finalProductPrices[$cliBranchProduct['CDPRODUTO']]['PRECO'] = (float)$this->formatPrice($finalProductPrices[$cliBranchProduct['CDPRODUTO']]['PRECO']);
            }

            foreach($clientTablePrices as $clientProduct){
                if(empty($finalProductPrices[$clientProduct['CDPRODUTO']])){
                    $finalProductPrices[$clientProduct['CDPRODUTO']] = $clientProduct;
                    $finalProductPrices[$clientProduct['CDPRODUTO']]['PRECO'] = (float)$this->formatPrice($finalProductPrices[$clientProduct['CDPRODUTO']]['PRECO']);
                }
            }

            foreach($storeTablePrices as $storeProduct){
                if(empty($finalProductPrices[$storeProduct['CDPRODUTO']])){
                    $finalProductPrices[$storeProduct['CDPRODUTO']] = $storeProduct;
                    $finalProductPrices[$storeProduct['CDPRODUTO']]['PRECO'] = (float)$this->formatPrice($finalProductPrices[$storeProduct['CDPRODUTO']]['PRECO']);
                }
            }

            foreach($branchTablePrices as $branchProduct){
                if(empty($finalProductPrices[$branchProduct['CDPRODUTO']])){
                    $finalProductPrices[$branchProduct['CDPRODUTO']] = $branchProduct;
                    $finalProductPrices[$branchProduct['CDPRODUTO']]['PRECO'] = (float)$this->formatPrice($finalProductPrices[$branchProduct['CDPRODUTO']]['PRECO']);
                }
            }

            foreach($generalTablePrices as $generalProduct){
                if(empty($finalProductPrices[$generalProduct['CDPRODUTO']])){
                    $finalProductPrices[$generalProduct['CDPRODUTO']] = $generalProduct;
                    $finalProductPrices[$generalProduct['CDPRODUTO']]['PRECO'] = (float)$this->formatPrice($finalProductPrices[$generalProduct['CDPRODUTO']]['PRECO']);
                }
            }

            $return = array(
				'error' => false,
				'prices' => $finalProductPrices
			);
        } catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao buscar produtos da tabela de precos. Erro: '.$e->getMessage()
			);
		}

        return $return;
    }
    
    public function getIdToken($CDFILIAL) {
        $sql = "
            SELECT IDTOKEN
              FROM INTEGRACAOFILI
             WHERE CDINTEGRACAO = '007'
               AND CDFILIAL = :CDFILIAL
        ";
        
        $params = array(
            ":CDFILIAL" => $CDFILIAL
        );


        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function setCdProdInte($CDPRODINTE, $CDPRODUTO, $CDPROJETO, $CDSUBPROJETO, $CDFILIAL) {
        $sql = "
            INSERT INTO PRODEXTERNOINTE
            (CDPRODINTE, CDPRODUTO, CDPROJETO, CDSUBPROJETO, CDFILIAL)
            VALUES
            (:CDPRODINTE, :CDPRODUTO, :CDPROJETO, :CDSUBPROJETO, :CDFILIAL)
        ";
        
        $params = array(
            ":CDPRODINTE" => $CDPRODINTE,
            ":CDPRODUTO" => $CDPRODUTO,
            ":CDPROJETO" => $CDPROJETO,
            ":CDSUBPROJETO" => $CDSUBPROJETO,
            ":CDFILIAL" => $CDFILIAL
        );


        $this->connection->executeQuery($sql, $params);
    }
    
    public function formatPrice($price) {
        $price = str_replace(',', '.', $price);
        return bcadd($price, '0.00', 2);
    }
    
    public function getCdSubProj($origin){
        $sql = "
            SELECT CDPROJETO, CDSUBPROJETO
              FROM CADASTROSUBPROJETO
             WHERE IDSUBPROJETO = :IDSUBPROJETO
        ";
        
        $params = [
            ":IDSUBPROJETO" => $origin
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getExternalCode($CDPRODUTO, $CDPROJETO, $CDSUBPROJETO, $CDFILIAL){
        $sql = "
            SELECT CDPRODINTE
              FROM PRODEXTERNOINTE
             WHERE CDPROJETO = :CDPROJETO
               AND CDSUBPROJETO = :CDSUBPROJETO
               AND CDPRODUTO = :CDPRODUTO
               AND CDFILIAL = :CDFILIAL
        ";
        
        $params = [
            ":CDPROJETO" => $CDPROJETO,
            ":CDSUBPROJETO" => $CDSUBPROJETO,
            ":CDPRODUTO" => $CDPRODUTO,
            ":CDFILIAL" => $CDFILIAL
        ];
        
        $result = $this->connection->fetchAssoc($sql, $params);
        
        return $result['CDPRODINTE'];
    }
    
    public function getErrorProducts($CDFILIAL){
        $sql = "
            SELECT CDPRODUTO
              FROM LOGEXPPROD
             WHERE CDFILIAL = :CDFILIAL
               AND IDTPEVENTO = 'F'
        ";
        
        $params = [
            ":CDFILIAL" => $CDFILIAL
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        $errorProducts = [];
        
        foreach($result as $errorProduct){
            if(isset($errorProduct['CDPRODUTO']))
                array_push($errorProducts, $errorProduct['CDPRODUTO']);
        }
        
        return $errorProducts;
    }
    
    public function updateErrorLog($CDPRODUTO, $CDFILIAL) {
        $sql = "
            UPDATE LOGEXPPROD
               SET IDTPEVENTO = 'R'
             WHERE CDPRODUTO = :CDPRODUTO
               AND CDFILIAL = :CDFILIAL
               AND IDTPEVENTO = 'F'
        ";
        
        $params = array(
            ":CDPRODUTO" => $CDPRODUTO,
            ":CDFILIAL" => $CDFILIAL
        );


        $this->connection->executeQuery($sql, $params);
    }

}
